/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.document.impl;

import java.io.InputStream;
import java.io.StringWriter;

import org.exoplatform.container.xml.InitParams;
import org.pdfbox.pdmodel.PDDocument;
import org.pdfbox.util.PDFTextStripper;

/**
 * Created by The eXo Platform SARL
 *
 * A parser of Adobe PDF files.
 * @author <a href="mailto:phunghainam@gmail.com">Phung Hai Nam</a>
 * @version Oct 19, 2005
 */
public class PDFDocumentReader extends BaseDocumentReader {

  /**
   * Initializes a newly created object for .pdf files format parsing.
   * @param params the container parameters.
  */
  public PDFDocumentReader(InitParams params) {
  }

  /**
   * Get the application/pdf mime type.
   * @return The application/pdf mime type.
   */
  public String getMimeTypes() {
    return "application/pdf";
  }

  /**
   * Returns only a text from .pdf file content.
   * @param is an input stream with .pdf file content.
   * @return The string only with text from file content.
   * @throws Exception
   */
  public String getContentAsText(InputStream is) throws Exception {
    PDDocument pdDocument = PDDocument.load(is);
    StringWriter sw = new StringWriter();
    try {
      PDFTextStripper stripper = new PDFTextStripper();
      stripper.setStartPage( 1 );
      stripper.setEndPage(Integer.MAX_VALUE );
      stripper.writeText(pdDocument, sw);
    } finally {
      if(pdDocument != null) pdDocument.close();
    }
    return sw.toString();
  }

}
