/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.templates.groovy;

import java.io.InputStream;

import org.exoplatform.container.component.BaseComponentPlugin;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Mar 2, 2005
 * @version $Id$
 * 
 * This class is used to search for and load  the template. The GroovyTemplateService
 * can have many GroovyTemplateLoader that load the template from the different locations.
 */
abstract public class GroovyTemplateLoader extends BaseComponentPlugin {
  final static public int VERY_HIGH_PRIORITY = 0 ;
  final static public int HIGH_PRIORITY = 1 ;
  final static public int NORMAL_PRIORITY = 2 ;
  final static public int LOW_PRIORITY = 3 ;
  final static public int VERY_LOW_PRIORITY = 4 ;
  
  private int priority_ = NORMAL_PRIORITY;
  
  /**
   * @return The  priority of this loader,  the lower  value,  the  higher priority
   */
  public int  getPriority() {  return priority_ ; }
  /**
   * @param prio  the new priority of  this loader
   */
  public void setPriority(int prio) { priority_ = prio ; }
  /**
   * This method  should be invoked  before  the load method  is called , Since the can load 
   * method can check the type  of the template and  it  can determine immediately that it can
   * load the template or not,  this will help  the loading process faster.
   * @param type  The type of the template, can be jar ,  file , database ...
   * @param name The name of  the template
   * @return  true  if the loader can load the template, alse  false
   * @throws Exception
   */
  abstract public boolean canLoad(String type, String name) throws Exception ;
  /**
   * This method should  read the content of the template and return as an InputStream 
   * @param name The name of the template
   * @return 
   * @throws Exception
   */
  abstract public InputStream load(String name) throws Exception ;
}
