/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.filter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.container.client.http.HttpClientInfo;
import org.exoplatform.portal.faces.component.PortalComponentCache;
import org.exoplatform.portal.session.PortalResources;
import org.exoplatform.portal.session.RequestInfo;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.services.portal.PortalACL;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.model.PortalConfig;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Sep 14, 2005
 */
public class RequestFact {
  private PortalContainer pcontainer_ ;
  private SessionContainer scontainer_ ;
  private HttpServletRequest request_ ;
  private HttpServletResponse response_ ;
  private RequestInfo  rinfo_ ; 
  private String  previousPage_ ;
  private boolean  firstRequest_ = false;
  private boolean  changeLink_ =  false ;
  private boolean  changePortal_ =  false ;
  private boolean  process_ =  true ;
  private String   errorMessage_  ;
  
  public RequestFact(PortalContainer pcontainer,
                     HttpServletRequest request, 
                     HttpServletResponse response ) throws Exception {
    pcontainer_ =  pcontainer ;
    request_ =  request ;
    response_ =  response ;
    HttpSession session = request_.getSession() ;     
    scontainer_ = pcontainer.getSessionManager().getSessionContainer(session.getId()) ;
    if(scontainer_ != null) {
      PortalResources res = 
        (PortalResources)scontainer_.getComponentInstanceOfType(PortalResources.class);
      request.setCharacterEncoding(res.getLocaleConfig().getInputEncoding()) ;
      
      firstRequest_ =  false ;
      rinfo_  = 
        (RequestInfo) scontainer_.getComponentInstanceOfType(RequestInfo.class);
      String previousAccessiblity =  rinfo_.getAccessibility() ;
      previousPage_ =  rinfo_.getNodeURI() ;
      rinfo_.init(request_) ;
      if(previousAccessiblity != rinfo_.getAccessibility()) changeLink_ =  true ;
      if(!rinfo_.getPortalOwner().equals(scontainer_.getOwner())) changePortal_ =  true ;
      rinfo_.setNewSession(false) ;
    } else  {
      firstRequest_ =  true ;
      rinfo_ = new RequestInfo() ;
      rinfo_.init(request_) ;
      rinfo_.setNewSession(true) ;
    }
  }
  
  public PortalContainer getPortalContainer() { return pcontainer_ ; }
  
  public SessionContainer getSessionContainer() { return scontainer_ ; }
  public void setSessionContainer(SessionContainer scontainer) { scontainer_ = scontainer ; }
  
  public HttpServletRequest getRequest() { return request_ ; }
  public HttpServletResponse getResponse() {  return response_ ; }
  
  public RequestInfo  getRequestInfo() { return rinfo_ ; }
  public String  getAccessLink() {  return rinfo_.getAccessibility(); }
  public String  getPreviousPage() {  return previousPage_ ; }
  
  public  boolean isNewRequest()  { return firstRequest_ ; }
  public  boolean isChangePortal() {  return changePortal_ ; }
  public  boolean isChangeLink() {  return changeLink_ ; }
  
  public boolean  getProcess() { return process_ ; }
  public void     setProcess(boolean b) {  process_  =  b ; }
  
  public  String  getErrorMessage() {  return errorMessage_ ; }
  public  void    setErrorMessage(String s) { errorMessage_ = s ; }
  
  public SessionContainer  newPortalSession() throws Exception {
    rinfo_ = new RequestInfo() ;
    rinfo_.init(request_) ;
    rinfo_.setNewSession(true) ;
    String owner  = rinfo_.getPortalOwner() ;
    HttpSession session = request_.getSession() ;
    Util.removeAttribute(session) ;
    pcontainer_.removeSessionContainer(session.getId()) ;
    scontainer_ =  pcontainer_.createSessionContainer(session.getId(), owner) ;
    scontainer_.setClientInfo(new HttpClientInfo(request_)) ;
    scontainer_.setUniqueId(owner +":"+rinfo_.getAccessibility()+":"+session.getId()) ;
    scontainer_.registerComponentInstance(RequestInfo.class, rinfo_);
    OrganizationService orgService = 
      (OrganizationService) pcontainer_.getComponentInstanceOfType(OrganizationService.class);
    UserProfile userProfile = 
      orgService.getUserProfileHandler().findUserProfileByName(scontainer_.getOwner());
    if (userProfile == null) { 
      userProfile = orgService.getUserProfileHandler().createUserProfileInstance();
    }
    scontainer_.registerComponentInstance(UserProfile.class, userProfile);
    PortalResources appres =  new PortalResources() ;
    scontainer_.registerComponentInstance(PortalResources.class, appres) ;
    scontainer_.registerComponentInstance(PortalComponentCache.class, new PortalComponentCache());
    return scontainer_ ;
  }
  
  public  boolean verifyPublicPortalAcess() throws Exception {
    PortalConfigService configService_ = 
      (PortalConfigService) pcontainer_.getComponentInstanceOfType(PortalConfigService.class) ;
    PortalConfig config = 
      configService_.getPortalConfig(rinfo_.getPortalOwner(), request_.getRemoteUser()) ;
    if(config != null) return true;
    pcontainer_.removeSessionContainer(request_.getSession().getId()) ;    
    PortalACL portalACL_ =  configService_.getPortalACL() ;
    if(!portalACL_.hasViewPortalPermission(config, request_.getRemoteUser())) {
      pcontainer_.removeSessionContainer(request_.getSession().getId()) ;
      errorMessage_ = "you cannot access the portal " +  rinfo_.getPortalOwner();
//      System .err.println("=====>>>>  do not have the portal permission access to portal '" + rinfo_.getPortalOwner() +"'") ;
//      System .err.println("=====>>>>  do not have the portal permission access to configuration '" + config.getOwner() +"'") ;
      return false ;  
    }  
    return true ;
  }
  
  public  boolean verifyPrivatePortalAcess() throws Exception {
    if(!rinfo_.getPortalOwner().equals(request_.getRemoteUser())) {
      pcontainer_.removeSessionContainer(request_.getSession().getId()) ;
      errorMessage_ = "you cannot access the private portal " +  rinfo_.getPortalOwner();
      return false ;
    }  
    return true ;
  }
  
  public  boolean verifyAdminPortalAcess() throws Exception {
    if(!request_.isUserInRole("admin")) {
      pcontainer_.removeSessionContainer(request_.getSession().getId()) ;
      errorMessage_ = "you cannot access the admin portal " +  rinfo_.getPortalOwner();
      return false ;
    }  
    return true ;
  }
}