package org.esupportail.cas.server.handlers.database;

import java.sql.Connection;

import org.dom4j.Element;
import org.esupportail.cas.server.util.RedundantHandler;

/**
 * This class implements a database server class, which can
 * authenticate users by binding to a database. It is used by 
 * BindDatabaseHandler.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr> 
 * @author Jean-Baptiste Daniel <danielj at users.sourceforge.net>
 */
public final class BindDatabaseServer extends DatabaseServer {
	
	/**
	 * Constructor.
	 *
	 * @param handlerDebug debugging mode of the handler
	 * @param handler the handler the server will be used by
	 * @param serverElement the XML element that declares the server 
	 * @throws Exception Exception
	 */
	public BindDatabaseServer(final Boolean handlerDebug,
							  final RedundantHandler handler,
							  final Element serverElement) throws Exception {
		super(handlerDebug, handler, serverElement);
		traceBegin();
		traceEnd();
	}
		
	/**
	 * Try to authenticate a user (by binding to the database).
	 *
	 * @param username the user's name
	 * @param password the user's password
	 *
	 * @return Server.AUTHENTICATE_SUCCESS, Server.AUTHENTICATE_NOAUTH
	 * or Server.AUTHENTICATE_FAILURE.
	 */	
	public int authenticate(
			final String username,
			final String password) {
		traceBegin();
		
		Connection connection = connect(username, password);
		
		switch (getConnectError()) {
			case CONNECT_SUCCESS:
				trace("Connection succeeded.");
				traceEnd("AUTHENTICATE_SUCCESS");
				return AUTHENTICATE_SUCCESS;
			case CONNECT_NOAUTH:
				trace("Connection refused.");
				traceEnd("AUTHENTICATE_NOAUTH");
				return AUTHENTICATE_NOAUTH;
			default:
				trace("Connection failure.");
			traceEnd("AUTHENTICATE_FAILURE");
			return AUTHENTICATE_FAILURE;
		}
	}
	
}

