package org.esupportail.cas.server.handlers.reject;

import org.dom4j.Element;
import org.esupportail.cas.server.util.BasicHandler;

/**
 * This class implements an reject handler handler class. It is used by
 * GenericHandler to exclude registred users.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 */
public final class RejectHandler extends BasicHandler {
	
	/**
	 * The id of the user that will be rejected.
	 */
	private String userId;
	
	/**
	 * Constructor.
	 *
	 * @param handlerElement the XML element that declares the handler 
	 * in the configuration file
	 * @param configDebug debugging mode of the global configuration
	 * @throws Exception Exception
	 */
	public RejectHandler(
			final Element handlerElement, 
			final Boolean configDebug) throws Exception {
		super(handlerElement, configDebug);
		traceBegin();

		checkConfigElement(true);
		
		userId = getConfigSubElementContent("username", true/*needed*/);
		trace("username = " + userId);

		traceEnd();
	}
	
	/**
	 * Try to authenticate a user.
	 *
	 * @param username the user's name
	 * @param password the user's password
	 *
	 * @return BasicHandler.FAILED_STOP if username matches,
	 * BasicHandler.FAILED_CONTINUE otherwise.
	 */	
	public int authenticate(
			final String username,
			final String password) {
		traceBegin();
		if (userId.equals(username)) {
			trace("Username matches");
			traceEnd("FAILED_STOP");
			return FAILED_STOP;
		} else {
			traceEnd("FAILED_CONTINUE");
			return FAILED_CONTINUE;
		}
	}
	
}

