/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.test.web.report;

import java.util.List;

import org.exoplatform.test.web.TestSuites;
import org.exoplatform.test.web.WebUnitMonitor;
import org.exoplatform.test.web.WebUnitSuite;
import org.exoplatform.test.web.unit.WebUnit;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Feb 20, 2006
 */
public class TextReport extends Report {
  final static private int NAME_COLUMN = 30 ;
  final static private int COUNTER_COLUMN = 8 ;
  final static private int STATUS_COLUMN = 8 ;  
  final static private int ERROR_COLUMN = 8 ;
  final static private int MALFORMED_COLUMN = 10 ;
  final static private int AVG_TIME_COLUMN = 10 ;
  final static private int CONTENT_LENGTH_COLUMN = 10 ;
  final static private int SUM_CONTENT_LENGTH_COLUMN = 10 ;
  
  protected boolean allStatus = false ;
  protected boolean brokenStatus = false ;
  protected boolean todoStatus = false ;
  
  public TextReport(TestSuites tsuites, String report) {
    super(tsuites) ;
    if(report == null) return ;
    String[] tmp = report.split(",") ;
    for(String status : tmp) {
      status = status.trim() ;
      if("broken".equals(status))  brokenStatus = true ;
      else if("todo".equals(status)) todoStatus = true;
      else if("all".equals(status)) allStatus = true;
    }
  }
  
  public void report() {
    StringBuilder b = new StringBuilder() ;
    b.append("***********************************************************************************\n") ;
    b.append(getSuitesSummary(tsuites_.getSuites())).append("\n\n") ;
    b.append(getUnitsSummary(tsuites_.getSuites())).append("\n\n") ;
    b.append("***********************************************************************************\n") ;
    System.out.print(b.toString()) ;
  }
  
  public String getSuitesSummary(List<WebUnitSuite> suites) {
    int NUMBER_COLUMN_WIDTH =  10 ;
    StringBuilder b = new StringBuilder(10000) ;
    appendColumn(b, "SUITE NAME", 40) ;
    appendColumn(b, "UNITS", NUMBER_COLUMN_WIDTH) ;
    appendColumn(b, "RUN", NUMBER_COLUMN_WIDTH) ;
    appendColumn(b, "TIME", NUMBER_COLUMN_WIDTH) ;
    appendColumn(b, "BROKEN", NUMBER_COLUMN_WIDTH) ;
    appendColumn(b, "TODO", NUMBER_COLUMN_WIDTH) ;
    b.append("\n\n") ; 
    int total = 0, runSum =0, timeSum = 0, brokenSum = 0, todoSum = 0 ;
    for(int i = 0 ; i < suites.size(); i++) {
      WebUnitSuite suite =  suites.get(i) ;      
      int run = countUnitRun(suite);
      int time = countAvgExecutionTime (suite);
      int broken  = countStatus(suite, WebUnit.STATUS_BROKEN) ;
      int todo = countStatus(suite, WebUnit.STATUS_INCOMPLETE) ;
      total += suite.getWebUnits().size() ;
      runSum +=run ;
      timeSum +=time ;
      brokenSum += broken ;
      todoSum += todo ;
      appendColumn(b, suite.getName(), 40) ;
      appendColumn(b, Integer.toString(suite.getWebUnits().size()), NUMBER_COLUMN_WIDTH) ;
      appendColumn(b, Integer.toString(run), NUMBER_COLUMN_WIDTH) ;
      appendColumn(b, Integer.toString(time), NUMBER_COLUMN_WIDTH) ;
      appendColumn(b, Integer.toString(broken), NUMBER_COLUMN_WIDTH) ;
      appendColumn(b, Integer.toString(todo), NUMBER_COLUMN_WIDTH) ;
      b.append("\n\n") ; 
    }
    appendColumn(b, "TOTAL", 40) ;
    appendColumn(b, Integer.toString(total), NUMBER_COLUMN_WIDTH) ;
    appendColumn(b, Integer.toString(runSum), NUMBER_COLUMN_WIDTH) ;
    appendColumn(b, Integer.toString(timeSum), NUMBER_COLUMN_WIDTH) ;
    appendColumn(b, Integer.toString(brokenSum), NUMBER_COLUMN_WIDTH) ;
    appendColumn(b, Integer.toString(todoSum), NUMBER_COLUMN_WIDTH) ;
    b.append("\n") ; 
    return b.toString() ;
  }
  
  public String getUnitsSummary(List<WebUnitSuite> suites) {
    StringBuilder b = new StringBuilder(10000) ;
    for(int i = 0 ; i < suites.size(); i++) {
      WebUnitSuite suite =  suites.get(i) ;
      if(printSuite(suite)) {
        b.append(getSummary(suite)).append("\n") ;
      }
    }
    return b.toString() ;
  }
  
  public String getSummary(WebUnitSuite suite) {
    List<WebUnit> units  = suite.getWebUnits() ;
    StringBuilder b = new StringBuilder(5000) ;
    StringBuilder berror = new StringBuilder(5000).append("BROKEN: \n") ;
    StringBuilder btodo = new StringBuilder(5000).append("TODO: \n") ;
    b.append("------------------------------------------------------------------------------------------\n");    
    b.append("Suite Name: ").append(suite.getName()).append("\n");
    b.append("Description: ").append(suite.getDescription()).append("\n");
    appendColumn(b, "Unit Name", NAME_COLUMN) ;
    appendColumn(b, "Status", STATUS_COLUMN) ;    
    appendColumn(b, "Counter", COUNTER_COLUMN) ;
    appendColumn(b, "Error", ERROR_COLUMN) ;
    appendColumn(b, "Malformed", MALFORMED_COLUMN) ;
    appendColumn(b, "Avg(ms)", AVG_TIME_COLUMN) ;
    appendColumn(b, "Avg(kb)", CONTENT_LENGTH_COLUMN) ;
    appendColumn(b, "Sum(kb)", SUM_CONTENT_LENGTH_COLUMN) ;        
    b.append("\n") ;     
    String unitStatus ;
    for(int i = 0 ; i < units.size(); i++) {            
      WebUnit unit =  units.get(i) ; 
      WebUnitMonitor monitor = unit.getMonitor() ;
      if(unit.getStatus() == WebUnit.STATUS_BROKEN) {
        berror.append("  name: ").append(unit.getName()).append("\n").
               append("  description: ").append(unit.getDescription()).append("\n").
               append("  comment: ").append(unit.getComment()).append("\n") ;
        unitStatus = "BROKEN" ;
      } else if(unit.getStatus() == WebUnit.STATUS_INCOMPLETE) {
        btodo .append("  name: ").append(unit.getName()).append("\n").
               append("  description: ").append(unit.getDescription()).append("\n").
               append("  comment: ").append(unit.getComment()).append("\n") ;
        unitStatus = "TODO" ;
      } else {
        unitStatus = "OK" ;
      }
      if(monitor.getCounter() == 0) continue ;
      appendColumn(b, unit.getName(), NAME_COLUMN) ; 
      appendColumn(b, unitStatus, STATUS_COLUMN) ;
      appendColumn(b, Integer.toString(monitor.getCounter()), COUNTER_COLUMN) ;       
      appendColumn(b, Integer.toString(monitor.getErrorCounter()), ERROR_COLUMN) ; 
      appendColumn(b, Integer.toString(monitor.getXhtmlMalformedCounter()), MALFORMED_COLUMN) ; 
      appendColumn(b, Long.toString(monitor.getAvgExecutionTime()), AVG_TIME_COLUMN) ; 
      appendColumn(b, Float.toString((float)monitor.getAvgContentLength()/1000), CONTENT_LENGTH_COLUMN) ; 
      appendColumn(b, Float.toString((float)monitor.getSumContentLength()/1000), SUM_CONTENT_LENGTH_COLUMN) ; 
      b.append("\n") ;
    }            
    return b.toString() + berror.toString() + btodo.toString() ;
  }
  
  private boolean printSuite(WebUnitSuite suite) {
    if(allStatus)  return true ;
    else if(brokenStatus && countStatus(suite, WebUnit.STATUS_BROKEN) > 0)  return true ;
    else if(todoStatus && countStatus(suite, WebUnit.STATUS_INCOMPLETE) > 0)  return true ;
    else return false ;
  }
  
  private void appendColumn(StringBuilder b, String value, int width) {
    b.append(value) ;
    for(int i = 0; i < width- value.length(); i++) {
      b.append(" ") ;
    }
  }
}
