/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.admin;

import condition.* ;
import suites.share.CommonUnit ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.share.UserLogoutSuite;

/**
 * Need to check that the changes to the rule have been taken into accoount 
 *
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class AdminNodeTypesSuite extends WebUnitSuite {
  static String DESC = "Go to Admin The ECM page, click on Manage Node Type button and test all the basic actions" ;
  static String NAME = "AdminNodeTypesSuite" ;
    
  public AdminNodeTypesSuite() {    
    super(NAME, DESC) ;   
    goToPage("/Home/ecm") ;
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", "Select Admin The ECM tab").
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageTemplates"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("Add/EditNodeType", "Click on Add/Edit Node Type button").
      setURL("&uicomponent=UIECMAdminSummary&op=addEditNodeType").       
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypeList&op=add"))      
    ) ;
    //Add node type
    addWebUnit(
      new ClickURLLinkUnit("CreateNodeType", "Click on add button").
      setURL("&uicomponent=UINodeTypeList&op=add").                  
      addValidator(new ExpectFormIdValidator("uiNodeTypeForm"))      
    ) ;
    //-----Add property
    addWebUnit(
      new SubmitFormUnit("AddProperty", "Click on Add property button").
      setFormId("uiNodeTypeForm").              
      setField("op","addProperty").                  
      addValidator(new ExpectFormIdValidator("uiPropertyDefinitionForm"))         
    ) ;    
    //----------Add constraints
    addWebUnit(
      new SubmitFormUnit("AddConstraints", "Click on add constraints button").
      setFormId("uiPropertyDefinitionForm").         
      setField("op","addConstraints").       
      addValidator(new ExpectFormIdValidator("uiSelectNodeType"))         
    ) ;
    
    addWebUnit(
      new SubmitFormUnit("SelectConstraints", "Select any constraints and click on save").
      setFormId("uiSelectNodeType").   
      setCheckBox("nt:nodeType", true).
      setField("op","save").              
      addValidator(new ExpectFormIdValidator("uiPropertyDefinitionForm")).
      addValidator(new ExpectTextValidator("[nt:nodeType]"))
    ) ;
    //----------Save property
    addWebUnit(
      new SubmitFormUnit("SaveProperty", "Click on add constraints button").
      setFormId("uiPropertyDefinitionForm").         
      setField("propertyname","exo:destWorkspace").      
      setField("requiredType","1").
      setField("autocreated","false").
      setField("mandatory","true").
      setField("parentversion","5").
      setField("protected","false").
      setField("multiple","false").
      setField("op","save").       
      addValidator(new ExpectFormIdValidator("uiNodeTypeForm"))         
    ) ;
    //-----Save node type
    addWebUnit(
      new SubmitFormUnit("SaveNodeType", "Click on save button").
      setFormId("uiNodeTypeForm").         
      setField("nodeTypeName","#{client.name}").      
      setField("mixinType","true").
      setField("hasOrderableChildNodes","false").
      setField("superTypes","nt:base").
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypeList&op=add"))         
    ) ;   
    //View node type
    addWebUnit(
      new ClickURLLinkUnit("ViewNodeType", "Click on view node type link").
      setURL("&uicomponent=UINodeTypeList&op=view&objectId=#{client.name}").                  
      addValidator(new ExpectFormIdValidator("uiNodeTypeForm")).
      addValidator(new ExpectTextValidator("#{client.name}")).
      addValidator(new ExpectTextValidator("false")).
      addValidator(new ExpectTextValidator("false")).
      addValidator(new ExpectTextValidator("nt:base")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypeForm&op=viewProperty&objectId=exo:destWorkspace"))
    ) ;
    //-----View property
    addWebUnit(
      new ClickURLLinkUnit("ViewProperty", "Click on exo:destWorkspace link").
      setURL("&uicomponent=UINodeTypeForm&op=viewProperty&objectId=exo:destWorkspace").                  
      addValidator(new ExpectFormIdValidator("uiPropertyDefinitionForm"))
    ) ;
    //----------Back
    addWebUnit(
      new SubmitFormUnit("ClickCancel", "Click on cancel button").
      setFormId("uiPropertyDefinitionForm").                  
      setField("op","PhaseId[1].cancel").
      addValidator(new ExpectFormIdValidator("uiNodeTypeForm"))
    ) ;
    //-----Back
    addWebUnit(
      new SubmitFormUnit("ClickCancel", "Click on cancel button").
      setFormId("uiNodeTypeForm").                  
      setField("op","PhaseId[1].cancel").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypeList&op=view&objectId=#{client.name}"))
    ) ;
    //Export node type
    addWebUnit(
      new ClickURLLinkUnit("ClickOnExport", "Click on export button").
      setURL("&uicomponent=UINodeTypeList&op=export").   
      addValidator(new ExpectFormIdValidator("uiNodeTypeExport"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("ClickCancel", "Click on cancel button").
      setFormId("uiNodeTypeExport").                  
      setField("op","PhaseId[1].cancel").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypeList&op=view&objectId=#{client.name}"))
    ) ;
    //Import node type
    addWebUnit(
      new ClickURLLinkUnit("ImportNodeType", "Click on import button").
      setURL("&uicomponent=UINodeTypeList&op=import").   
      addValidator(new ExpectFormIdValidator("uiNodeTypeUpload"))
    ) ;
    //-----File name not found!(file name is blank)
    def is;
    is = Thread.currentThread().
        getContextClassLoader().
        getResourceAsStream("") ;        
    addWebUnit(
      new SubmitFormUnit("UploadBlankFilename", "Upload file with blank file name").
      setFormId("uiNodeTypeUpload").        
      setField("op","import").
      addValidator(new ExpectTextValidator("File name not found!"))      
    ) ;
    //-----File format ERROR. Must be import .xml file!(file name is "abc" style, or isn't in style *.xml)
    is = Thread.currentThread().
        getContextClassLoader().
        getResourceAsStream("org/exoplatform/test/web/Tasks.class") ;        
    addWebUnit(
      new SubmitFormUnit("UploadWrongFormatFile", "Upload file that file name isn't in *.xml").
      setFormId("uiNodeTypeUpload").   
      setFileField("upload","Tasks.class", is).
      setField("op","import").
      addValidator(new ExpectTextValidator("File format ERROR. Must be import .xml file!"))      
    ) ;
    /*
    //-----Data import ERROR. Data in import file isn't valid nodetype's format!
    //-----Data format is OK
    is = Thread.currentThread().getContextClassLoader().getResourceAsStream("nodetypeexport.xml");
    print "===========================> is : " + is ;
    addWebUnit(
      new SubmitFormUnit("UploadIsOK", "Upload file valid file name").
      setFormId("uiNodeTypeUpload").   
      setFileField("upload","nodetypeexport.xml", is).
      setField("op","import").
      addValidator(new ExpectFormIdValidator("uiNodeTypeImport"))      
    ) ;
    /*
    addWebUnit(
      new ClickURLLinkUnit("ClickOnImport", "Click on import button").
      setURL("&uicomponent=UINodeTypeList&op=import").   
      addValidator(new ExpectFormIdValidator("uiNodeTypeExport"))
    ) ;
    */
    //Back to Admin the ECM
    addWebUnit(
      new SubmitFormUnit("ClickOnCancel", "Click on cancel button").
      setFormId("uiNodeTypeUpload").
      setField("op","PhaseId[1].cancel").        
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypeList&op=import"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBack", "Click on back button").
      setURL("uicomponent=UINodeTypeList&op=back").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIECMAdminSummary&op=addEditNodeType"))
    ) ;
  }     
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }  
}