/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;


/**
 * This suite depends on the AddFolderSuite one as the document is added in thee
 * /test-folder
 *     1) Go to the test-folder
 *     2) Test the document type selection
 *     3) Add new File document
 *     4) Save session
 *     5) Return to workspace selection
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class UserAddDocumentSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page and add a new document to test-folder" ;
  static String NAME = "UserAddDocumentSuite" ;
    
  public UserAddDocumentSuite() {    
    super(NAME, DESC) ;
    addWebSuite(new UserAddFolderSuite());        
    //Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ;
    //Click on Add document
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectDocumentForm")).
      addValidator(new ExpectFormIdValidator("documentForm"))
    ) ;
    //-----Test the document type selection
    addWebUnit(
      new SubmitFormUnit("SelectDocumentType", "select document type is exo:article").
      setFormId("selectDocumentForm").   
      setField("documentType","exo:article").
      setField("op","changeDocument").
      addValidator(new ExpectTextValidator("Image to upload"))
    ) ;
    //-----Add new Nt:file document
    addWebUnit(
      new SubmitFormUnit("ReselectDocumentType", "select document type is nt:file").
      setFormId("selectDocumentForm").   
      setField("documentType","nt:file").
      setField("op","changeDocument").
      addValidator(new ExpectNoTextValidator("Image to upload")).
      addValidator(new ExpectTextValidator("File")).
      addValidator(new ExpectTextValidator("Content:"))
    ) ;
    //----------Save New Document
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("documentForm").   
      setField("docName","test-doc").
      setField("summary","content").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=editDocument&objectId=test-doc"))
    ) ;                
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    //Return to workspace selection
    addWebUnit(
      new ClickURLLinkUnit("SelectWorkspace", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectWorkspace").                  
      addValidator(new ExpectTextValidator("Workspaces "))      
    ) ;  
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}