/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.portal;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import org.exoplatform.test.portlet.Login.*;

/**
 * This suite does:
 *    1)Go to browse page and delete if existed newpage
 *    2)Add a new page
 *    3)Add portlet to page
 *    4)Save action add newpage
 *    5)Go to list page
 *    
 * Created by The eXo Platform SARL
 * Author : Hoa  Nguyen Thi
 *          hoa2501@yahoo.com
 * Mar 23, 2006
 */
public class AddPageBrowseSuite extends WebUnitSuite {
  String COMMENT="You must run PortletRegistrySuite suite before you run this suite";
  public AddPageBrowseSuite(String username,String categoryPortlet,String portletname,String viewPermission,String editPermission) {
    super("AddPageBrowseSuite", "Click browse page and add a newpage") ;
    addWebUnit(
        new ClickURLLinkUnit("DeleteExistedNewPage", "Delete if existed NewPage").
        setURL("uicomponent=UIPageList&op=delete&objectId=${username}:newPage").
        addCondition(new HaveLinkWithURLCondition("uicomponent=UIPageList&op=delete&objectId=${username}:newPage")).
        addValidator(new ExpectTextValidator("The page has been deleted successfully")) 
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("SaveActionDelete", "Save action delete").
        setURL("uicomponent=UIToolbarNav&op=saveNode").
        addValidator(new ExpectLinkWithURLValidator("${username}?uicomponent=UIToolbar&op=newPage"))
    ) ;
    //-----Add a new page with view and edit permission is guest
    addWebUnit(
        new ClickURLLinkUnit("ClickAddNewPage", "Click on New Page").
        setURL(username+"?uicomponent=UIToolbar&op=newPage").
        addValidator(new ExpectFormIdValidator("pageModelForm")).
        addValidator(new ExpectTextValidator("New Page")) 
    ) ; 
    addWebUnit(
        new SubmitFormUnit("CreatenewPage", "Fill page's information and create this page").
        setFormId("pageModelForm").
        setField("name","newPage").
        setField("template", "2-columns-page").
        setField("op", "save").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=preview&objectId=${username}:newPage")) 
    ); 
    addWebUnit(
        new ClickURLLinkUnit("ClickPreview", "Click on Preview icon").
        setURL("uicomponent=UIPageList&op=preview&objectId=${username}:newPage").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPagePreviewToolbar&op=edit"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickEdit", "Click on Edit icon").
        setURL("uicomponent=UIPagePreviewToolbar&op=edit").
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=page&portal:action=editProperty"))
    ) ;
    //----------Add a new Portlet to newPage
    addWebUnit(
        new ClickURLLinkUnit("SelectContentCategory", "Click on Add Portlet icon").
        setURL("portal:componentId=page&portal:action=addPortlet").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPortletSelecttor&op=selecPortlet&objectId="))
    ) ;
    addWebSuite(new AddPortletSuite(categoryPortlet,portletname));
    addWebUnit(
        new ClickURLLinkUnit("ClickOnSave", "Click on save icon").
        setURL("uicomponent=UIPagePreviewToolbar&op=save").
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=page&portal:action=addPortlet"))
    ) ;
    //----------Set view and edit permission is any guest for newPage
    addWebUnit(
        new ClickURLLinkUnit("ClickEditPageProperties", "Click on Edit Page Properties icon").
        setURL("portal:componentId=page&portal:action=editProperty").
        addValidator(new ExpectFormIdValidator("pageForm"))
    ) ; 
    addWebUnit(
        new SubmitFormUnit("EditPageProperties", "Edit some page properties ").
        setFormId("pageForm").        
        setField("title", "newPage").
        setField("viewPermission",viewPermission).
        setField("editPermission",editPermission).
        setField("renderer", "PageColumnRenderer").
        setField("decorator","default").
        setField("op","save").
        addValidator(new ExpectTextValidator("newPage"))
    );
    addWebUnit(
        new ClickURLLinkUnit("SaveEditPage", "Click on Save Edit Page Properties icon").
        setURL("uicomponent=UIPagePreviewToolbar&op=save").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPagePreviewToolbar&op=PhaseId[1].cancel"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickCancelIcon", "Click cancel to go to list page").
      setURL("uicomponent=UIPagePreviewToolbar&op=PhaseId[1].cancel").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=preview&objectId=${username}:newPage"))
    ) ; 
  }
}