/**************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved. *
 * Please look at license.txt in info directory for more license detail.  *
 **************************************************************************/
package suites.portal;

import condition.* ;
import suites.share.* ;
import suites.share.UserLoginSuite;
import suites.share.UserLogoutSuite;

import org.exoplatform.test.*;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * This suite does :
 *    1)Create a new page with view and edit permission is any admin
 *    2)Create a new child node of SiteMap node on Homepage with 
 *      page reference is newpage and view and edit permission is any guest
 *    3)Check some permission for page
 *        3.1)Check view  permission is any admin
 *        3.2)Check view  permission is owner
 *        3.3)Check view  permission is any user
 *        3.4)Check view  permission is any guest
 *    4)Check some permission for node
 *        4.1)Check view  permission is any admin
 *        4.2)Check view  permission is any guest
 *        4.3)Check view  permission is owner
 *    5)Delete created child node of SiteMap
 *    6)Delete created new page   
 *    
 * Created by The eXo Platform SARL
 * Author : Hoa  Nguyen Thi
 *          hoa2501@yahoo.com
 * Mar 31, 2006
 */
public class ExoCustomizationNavSuite extends WebUnitSuite {
  String COMMENT  = "You must run PortletRegistrySuite suite before run this suite";
  String[] roles=["user"];
  public ExoCustomizationNavSuite() {
    super("ExoCustomizationNavSuite", "Test some permission for add new node of sitemap") ;
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    //Go to Browse Page
    addWebUnit(CommonUnit.CLICK_BROWSE_PAGES_NAV) ;
    addWebSuite(new AddPageBrowseSuite("exo","content","SearchPortlet","*:/admin","*:/admin")) ;
    //Go to Show Navigation
    addWebUnit( CommonUnit.CLICK_SHOW_NAV) ;
    addWebUnit(
        new ClickURLLinkUnit("CheckExistChildNode", "Click '+' on SiteMap").
        setURL("uicomponent=UINavigationManager&op=selectNode&objectId=/home/sitemap").
        addCondition(new HaveLinkWithURLCondition("uicomponent=UINavigationManager&op=selectNode&objectId=/home/sitemap")).
        addValidator(new ExpectTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("DeleteExistedChildNode", "Delete if existed ChildNode").
        setURL("uicomponent=UINavigationManager&op=deleteNode&objectId=/home/sitemap/ChildNode").
        addCondition(new HaveLinkWithURLCondition("uicomponent=UINavigationManager&op=deleteNode&objectId=/home/sitemap/ChildNode")).
        addValidator(new ExpectNoTextValidator("ChildNode"))
    ) ;  
    //-----Create ChildNode on SiteMap with permission is guest
    addWebUnit(
      new ClickURLLinkUnit("ClickAddNode(onSiteMap)", "Click 'add' on sitemap").
      setURL("uicomponent=UINavigationManager&op=addNode&objectId=/home/sitemap").
      addValidator(new ExpectFormIdValidator("pageNodeForm")).
      addValidator(new ExpectTextValidator("Add/Edit Node")) 
    ) ; 
    addWebSuite(new SearchPermissionSuite("pageNodeForm","View",
       "PhaseId[1].searchGroupForViewPerm","*","guest"));    
    addWebSuite(new SearchPermissionSuite("pageNodeForm","Edit",
                     "PhaseId[1].searchGroupForEditPerm", "*" ,"guest"));
    addWebUnit(
        new SubmitFormUnit("SearchPageReference", "Click Search Page Reference icon").
        setFormId("pageNodeForm").
        setField("op","PhaseId[1].searchXhtmlRefPage").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=select&objectId=exo:newPage"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SelectPageReference", "Select Page Reference").
        setURL("uicomponent=UIPageList&op=select&objectId=exo:newPage").
        addValidator(new ExpectFormIdValidator("pageNodeForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("Create'ChildNode'", "Create a child node with some properties are default").
      setFormId("pageNodeForm").
      setField("name","ChildNode").
      setField("displayName","ChildNode").
      setField("op","save").
      addValidator(new ExpectLinkWithTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("CheckExistChildNode", "Click '+' on SiteMap").
        setURL("uicomponent=UINavigationManager&op=selectNode&objectId=/home/sitemap").
        addValidator(new ExpectTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SaveChildNode", "Click Save icon").
        setURL("&uicomponent=UIToolbarNav&op=saveNode").
        addValidator(new ExpectTextValidator("ChildNode"))
    ) ;
    
    //Check view and edit permission of newPage is any admin
    addWebSuite(new UserLogoutSuite());
    addWebUnit(
        new ClickLinkWithText("GoToChildNode", "Go to ChildNode on SiteMap").
        setTextLink("ChildNode").
        addValidator(new ExpectNoTextValidator("SearchPortlet"))        
    ) ;
    roles = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles));
    addWebUnit(
        new ClickLinkWithText("SelectPortalAdmin", "Go to site portal admin").
        setTextLink("Portal Admin").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIAvailablePortal&op=admin&objectId=exo"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("GoToExo", "Select Domain Exo to check view permission Node for admin  ").
        setURL("uicomponent=UIAvailablePortal&op=admin&objectId=exo").
        addValidator(new ExpectLinkWithTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickLinkWithText("GoToChildNode", "Go to ChildNode on SiteMap to check permission view page for admin").
        setTextLink("ChildNode").
        addValidator(new ExpectLinkWithURLValidator("windowState=minimized"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("BackToAdmin", "Back to Domain of  Admin ").
        setURL("admin:returnAction=return").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UILogout&op=logout"))
    ) ;
    addWebSuite(new UserLogoutSuite());   
  
    //Check view and edit permission of newPage is owner
    addWebSuite(new UserLoginSuite("exo", "exo", roles)) ;
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_BROWSE_PAGES_NAV) ;
    addWebSuite(new EditPageBrowseSuite("exo","newPage","pageForm","newPage","owner:/user","owner:/user")) ;
    addWebSuite(new UserLogoutSuite());  
    addWebSuite(new UserLoginSuite("demo", "exo", roles));
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_BROWSE_PAGES_NAV) ;
    addWebUnit(
        new SubmitFormUnit("SearchPageOfExo", "Click Search icon").
        setFormId("searchForm").
        setField("owner","exo").
        setField("op","searchPages").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=select&objectId=exo:newPage"))
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("SelectPageReference", "Select Page Reference").
        setURL("uicomponent=UIPageList&op=select&objectId=exo:newPage").
        addValidator(new ExpectTextValidator("You need to have the role onwer to execute the action selectPage"))
    ) ;
    addWebSuite(new UserLogoutSuite());  
    
    //Check view and edit permission of newPage is any user
    addWebSuite(new UserLoginSuite("exo", "exo", roles)) ;
    addWebUnit(CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_BROWSE_PAGES_NAV) ;
    addWebSuite(new EditPageBrowseSuite("exo","newPage","pageForm","newPage","*:/user","*:/user")) ;
    addWebSuite(new UserLogoutSuite());
    addWebSuite(new UserLoginSuite("demo", "exo", roles));
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit( CommonUnit.CLICK_SHOW_NAV) ;
    addWebUnit(
        new ClickURLLinkUnit("DeleteExistedChildNode", "Delete if existed ChildNode").
        setURL("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode").
        addCondition(new HaveLinkWithURLCondition("uicomponent=UINavigationManager&op=deleteNode&objectId=/ChildNode")).
        addValidator(new ExpectNoTextValidator("ChildNode"))
    ) ;  
    //-----Create new node on HomePage with permission is any user
    addWebUnit(
      new ClickURLLinkUnit("ClickAddNode(onHomePage)", "Click 'add' on homepage").
      setURL("uicomponent=UINavigationManager&op=addNode&objectId=/").
      addValidator(new ExpectFormIdValidator("pageNodeForm")).
      addValidator(new ExpectTextValidator("Add/Edit Node")) 
    ) ; 
    addWebSuite(new SearchPermissionSuite("pageNodeForm","View",
       "PhaseId[1].searchGroupForViewPerm","*","guest"));    
    addWebSuite(new SearchPermissionSuite("pageNodeForm","Edit",
                     "PhaseId[1].searchGroupForEditPerm", "*" ,"guest"));
    addWebUnit(
        new SubmitFormUnit("SearchPageReference", "Click Search Page Reference icon").
        setFormId("pageNodeForm").
        setField("op","PhaseId[1].searchXhtmlRefPage").
        addValidator(new ExpectFormIdValidator("searchForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SearchPageOfExo", "Click Search icon").
        setFormId("searchForm").
        setField("owner","exo").
        setField("op","searchPages").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=select&objectId=exo:newPage"))
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("SelectPageReference", "Select Page Reference").
        setURL("uicomponent=UIPageList&op=select&objectId=exo:newPage").
        addValidator(new ExpectFormIdValidator("pageNodeForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("Create'ChildNode'", "Create a child node with some properties are default").
      setFormId("pageNodeForm").
      setField("name","ChildNode").
      setField("displayName","ChildNode").
      setField("op","save").
      addValidator(new ExpectLinkWithTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SaveChildNode", "Click Save icon").
        setURL("&uicomponent=UIToolbarNav&op=saveNode").
        addValidator(new ExpectTextValidator("ChildNode"))
    ) ;   
    addWebUnit(
        new ClickLinkWithText("ClickChildNodeButton", "Click on ChildNode button of navigation ").
        setTextLink("ChildNode").
        addValidator(new ExpectTextValidator("SearchPortlet"))
    ) ;   
    addWebSuite(new UserLogoutSuite());    
    
    //Check view and edit permission of newpage is any guest
    addWebSuite(new UserLoginSuite("exo", "exo", roles)) ;
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_BROWSE_PAGES_NAV) ;
    addWebSuite(new EditPageBrowseSuite("exo","newPage","pageForm","newPage","*:/guest","*:/guest")) ;
    addWebSuite(new UserLogoutSuite());
    addWebUnit(
      new ClickLinkWithText("GoToChildNode", "Go to ChildNode on SiteMap").
      setTextLink("ChildNode").
      addValidator(new ExpectLinkWithURLValidator("portletMode=help"))
    ) ; 
    
    //Check view and edit permission of childNode is any admin
    addWebSuite(new UserLoginSuite("exo", "exo", roles)) ;
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_SHOW_NAV) ;
    addWebSuite(new ExoEditNodeSuite("home/sitemap","ChildNode","pageNodeForm","*:/admin","*:/admin"));
    addWebSuite(new UserLogoutSuite());
    roles = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles));
    addWebUnit(
        new ClickLinkWithText("SelectPortalAdmin", "Go to site portal admin").
        setTextLink("Portal Admin").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIAvailablePortal&op=admin&objectId=exo"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("GoToExo", "Select Domain Exo to check view permission Node for admin  ").
        setURL("uicomponent=UIAvailablePortal&op=admin&objectId=exo").
        addValidator(new ExpectLinkWithTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickLinkWithText("GoToChildNode", "Go to ChildNode on SiteMap to check permission view page for admin").
        setTextLink("ChildNode").
        addValidator(new ExpectLinkWithURLValidator("portletMode=help"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("BackToAdmin", "Back to Domain of  Admin ").
        setURL("admin:returnAction=return").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UILogout&op=logout"))
    ) ;
    addWebSuite(new UserLogoutSuite());   
    
    //Check view and edit permission of childNode is any guest
    roles = ["user"] ;
    addWebSuite(new UserLoginSuite("exo", "exo", roles)) ;
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_SHOW_NAV) ;
    addWebSuite(new ExoEditNodeSuite("home/sitemap","ChildNode","pageNodeForm","*:/guest","*:/guest"));
    addWebUnit(
        new ClickURLLinkUnit("CheckViewNodeForGuest", "Logout then check guest must not view ChildNode").
        setURL("&uicomponent=UILogout&op=logout").        
        addValidator(new ExpectFormIdValidator("loginForm")).
        addValidator(new ExpectLinkWithTextValidator("ChildNode"))
    ) ;     
    
    //Check view and edit permission of childNode is owner
    addWebSuite(new UserLoginSuite("exo", "exo", roles)) ;
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_SHOW_NAV) ;
    addWebSuite(new ExoEditNodeSuite("home/sitemap","ChildNode","pageNodeForm","owner:/user","owner:/user"));
    addWebUnit(
        new ClickURLLinkUnit("CheckViewNodeForGuest", "Logout then check guest must not view ChildNode").
        setURL("&uicomponent=UILogout&op=logout").        
        addValidator(new ExpectFormIdValidator("loginForm")).
        addValidator(new ExpectLinkWithTextValidator("ChildNode",0))
    ) ; 
    roles = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles));
    addWebUnit(
        new ClickLinkWithText("SelectPortalAdmin", "Go to site portal admin").
        setTextLink("Portal Admin").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIAvailablePortal&op=admin&objectId=exo"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("GoToExo", "Select Domain Exo to check view permission Node for admin  ").
        setURL("uicomponent=UIAvailablePortal&op=admin&objectId=exo").
        addValidator(new ExpectLinkWithTextValidator("ChildNode",0))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("BackToAdmin", "Back to Domain of  Admin ").
        setURL("admin:returnAction=return").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UILogout&op=logout"))
    ) ;
    
    //Go to account exo to delete page and node    
    addWebSuite(new UserLoginSuite("exo", "exo", roles)) ;
    addWebUnit(CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_BROWSE_PAGES_NAV) ;
    //-----Delete  existed newpage
    addWebUnit(
        new ClickURLLinkUnit("DeleteNewPage", "Delete existed NewPage").
        setURL("uicomponent=UIPageList&op=delete&objectId=exo:newPage").
        addValidator(new ExpectTextValidator("The page has been deleted successfully")) 
    ) ; 
    //Go to Show Navigation
    addWebUnit( CommonUnit.CLICK_SHOW_NAV) ;
    addWebUnit(
        new ClickURLLinkUnit("CheckExistChildNode", "Click '+' on SiteMap").
        setURL("uicomponent=UINavigationManager&op=selectNode&objectId=/home/sitemap").
        addCondition(new HaveLinkWithURLCondition("uicomponent=UINavigationManager&op=selectNode&objectId=/home/sitemap")).
        addValidator(new ExpectTextValidator("ChildNode"))
    ) ;
    //-----Delete existed childNode
    addWebUnit(
        new ClickURLLinkUnit("DeleteExistedChildNode", "Delete existed childNode").
        setURL("uicomponent=UINavigationManager&op=deleteNode&objectId=/home/sitemap/ChildNode").
        addCondition(new HaveLinkWithURLCondition("uicomponent=UINavigationManager&op=deleteNode&objectId=/home/sitemap/ChildNode")).
        addValidator(new ExpectNoTextValidator("ChildNode"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnSave", "Click on save icon").
        setURL("uicomponent=UIToolbarNav&op=saveNode").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbarNav&op=saveNode"))
    ) ;
  }  
  public java.util.List getRequiredSuites() {         
    String[] roles = ["user"] ;    
    return [new UserLoginSuite("exo", "exo", roles)] ;  
  } 
}
